***** Make Friends with SAS, Day 3: Increasing Efficiency with ARRAYS, LOOPS, and MACROS;
***** Created by Lesa Hoffman;

* Output options: all can be turned on/off by adding or removing the "NO":
  page number, date, centering, or page breaks, page length and width; 
OPTIONS nonumber nodate nocenter formdlim='-' pagesize=MAX linesize=120;
TITLE;   * Shut off default SAS titles;


*******************************************************************************
******* 					 	DO LOOPS						        *******
******************************************************************************;

* DO loops can be used in DATA steps to do something repeatedly.
  Here, and example making up make data (i.e., simulation).
  The RAND functions create variables that fit certain distributions;

* Simulate cross-sectional data;
DATA fakepeople;   * No SET statement because data dont exist yet;
	CALL STREAMINIT(8675309); 		* Set random number seed for RAND functions;
	DO PersonID=1 TO 500 BY 1;			* Start making 500 people;
		NormalVar = RAND("NORMAL",0,1); * Make normal variable with mean=0, SD=1;
		CountVar  = RAND("POISSON",3);  * Count variable with mean=var=3;
		UniformVar = RAND("UNIFORM");   * Uniform variable ranging from 0 to 1;
		OUTPUT;						    * Write that row of data with 4 variables;
	END;  * END shuts off DO;
RUN;

* Now lets see what it made;
TITLE1 "Our New Variable Distributions!";
PROC UNIVARIATE NOPRINT DATA=fakepeople;
	VAR NormalVar CountVar UniformVar;
	HISTOGRAM NormalVar CountVar UniformVar;
RUN; TITLE1;

* Simulate longitudinal data with 2 nested DO LOOPS;
DATA longfakepeople;   * No SET statement because data dont exist yet;
	CALL STREAMINIT(8675309); 		* Set random number seed for RAND functions;
	DO PersonID=1 TO 50 BY 1;				* Start making 50 people;
		DO Time=1 TO 5 BY 1;				* Start making 5 occasions;
			NormalVar = RAND("NORMAL",0,1); * Make normal variable with mean=0, SD=1;
			CountVar  = RAND("POISSON",3);  * Count variable with mean=var=3;
			UniformVar = RAND("UNIFORM");   * Uniform variable ranging from 0 to 1;
			OUTPUT;						    * Write that row of data with 4 variables;
		END;  * END shuts off Time DO;
	END;	  * END shuts off Person DO;
RUN;
		


*******************************************************************************
******* 					ARRAYS WITH DO LOOPS						*******
******************************************************************************;

* ARRAY is a short-hand for a list of variables
  ARRAY names must not duplicate any variable names
  Parentheses after array name is #variables, followed by variable list:
  Arraynam(1)=var1, arraynam(2)=var2, etc;

* Here is an example that gets rid of missing values in a series;
DATA &datafile.; SET &datafile.;
    ARRAY anew (100) newvar1 -  newvar100;  * Ordered by name;
    ARRAY aold (100) oldvar1 -- oldvar100;  * Ordered in dataset;
    DO i = 1 TO 100 BY 1;		* DO loop says do what follows 100 times;
		* If old has missing data, change new to system missing;
    	  IF aold(i) = 999 THEN anew(i) = .;
		* Otherwise, copy old value;
          ELSE anew(i) = aold(i);
    END; * END is needed to shut off DO;
	DROP i;  * Counting variable is removed from data;
RUN;

* Your turn;

* Define location of file;
%LET filepath = F:\13_Psyc930SAS\Day3_Array_DO_Macros;
* Import GRI_data.sav;
PROC IMPORT DATAFILE = "&filepath.\GRI_data.sav" OUT=work.GRI_data DBMS=SAV REPLACE;
RUN;
* Strip out formats to variables appear as numeric;
DATA work.GRI_data; SET work.GRI_data;
	FORMAT _ALL_;
RUN;
* Recoding -99 into missing, AND recoding all items;
DATA work.GRI_data; SET work.GRI_data;
	* List of new variables to be created;
	  ARRAY anew (24) newgps1-newgps24;
	* List of old variables to be recoded (in order);
	  ARRAY aold (24) gps1--gps24;
	* Do this 24 times, once per item on the list;
	  DO i=1 TO 24;
	         IF aold(i)=. OR aold(i)=-99 THEN anew(i)=.;
		ELSE IF aold(i)=6 THEN anew(i)=1;
		ELSE IF aold(i)=5 THEN anew(i)=2;
		ELSE IF aold(i)=4 THEN anew(i)=3;
		ELSE IF aold(i)=3 THEN anew(i)=4;
		ELSE IF aold(i)=2 THEN anew(i)=5;
		ELSE IF aold(i)=1 THEN anew(i)=6;
		ELSE anew(i)=.;
	END; DROP i;
RUN;




*******************************************************************************
******* 			MACRO PROGRAMS THAT WRITE SAS CODE					*******
******************************************************************************;

* MACRO options for debugging macro programs;
OPTIONS merror serror mprint mlogic symbolgen spool; 

* So far we have seen macro variables. Now we will see macro programs!
  Macro commands begin with %, such as we have seen so far;
%LET filepath=F:\13_Psyc930SAS\Day3_Array_DO_Macros;
%LET dataname=work.GRI_data;


************************************************************************************;
* Other macro commands can iteratively write SAS code, such as in DO LOOPS;
* Lets write a macro program that adds variable labels to our 24 new variables;
************************************************************************************;

* Now we need 24 variable labels -- make SAS do it;
%MACRO MakeMyLabels;
	DATA &dataname.; SET &dataname.;
		%DO num=1 %TO 24;
			LABEL newgps&num.= "New GPS &num. recoded";
		%END;
	RUN;
%MEND MakeMyLabels;
* NOW GO Make My Labels! ;
%MakeMyLabels;



************************************************************************************;
* Now lets write a program that systematically replaces sample sizes
  in our longitudinal fake people, and adds reps per condition;
************************************************************************************;

%MACRO MakePeople(Npersons, Ntimes);
	* Simulate longitudinal data with 2 nested DO LOOPS;
	DATA Npersons&Npersons._Ntimes&Ntimes.;   * No SET statement because data dont exist yet;
		CALL STREAMINIT(0); 				  * Seed is now completely random (from clock);
		DO Rep=1 TO 500 BY 1;					* Start making 500 replications;
			DO PersonID=1 TO &Npersons. BY 1;		* Start making 50 people;
				DO Time=1 TO &Ntimes. BY 1;			* Start making 5 occasions;
					NormalVar = RAND("NORMAL",0,1); * Make normal variable with mean=0, SD=1;
					CountVar  = RAND("POISSON",3);  * Count variable with mean=var=3;
					UniformVar = RAND("UNIFORM");   * Uniform variable ranging from 0 to 1;
					OUTPUT;						    * Write that row of data with 5 variables;
				END;  * END shuts off Time DO;
			END;	  * END shuts off Person DO;
		END;		  * END shuts off Rep DO;
	RUN;
%MEND MakePeople;
* Now make data	files for each simulation condition;
%MakePeople(Npersons=10, Ntimes=3);
%MakePeople(Npersons=50, Ntimes=3);
%MakePeople(Npersons=10, Ntimes=6);
%MakePeople(Npersons=50, Ntimes=6);


************************************************************************************;
* Now lets a write a program that systematically imports multiple csv data files
  and merges them all into a single SAS data file;
************************************************************************************;

* Location for csv files to import;
  %LET folder=F:\13_Psyc930SAS\Day3_Array_DO_Macros;
* Range of file numbers;
  %LET start = 128;
  %LET end = 130;
* Name of final combined file to be saved;
  %LET permname = work.CSVMerged;

* Creating empty data file to augment with each csv file;
DATA &permname.; RUN;

* Importing each excel file, starting with # for start and ending with # for end;
* Any file numbers missing in the sequence will generate no data sets (ignore errors);
%MACRO ImportCSV;
	%DO name=&start. %TO &end.;
		PROC IMPORT OUT=work.csv&name. DATAFILE="&folder.\&name..csv" DBMS=CSV REPLACE; 
		     GETNAMES=YES; DATAROW=2; 
		RUN;
		* Now add to previous files (will be empty first time through);
		DATA &permname.; 
			SET &permname. work.csv&name.; 
		RUN;
	%END;
%MEND ImportCSV;
* Now call the macro to import all files from "start" to "end";
%ImportCSV;


************************************************************************************;
* Now lets a write a program that systematically imports multiple excel worksheets
  and merges them all into a single SAS data file, adding a subject variable;
************************************************************************************;

* Location for csv files to imported;
  %LET folder=F:\13_Psyc930SAS\Day3_Array_DO_Macros;
* Master excel file name to import;
  %LET excelname=Subjects.xlsx;
* Range of file numbers;
  %LET start = 128;  
  %LET end = 130;
* Name of final combined file to be saved;
  %LET permname = work.ExcelMerged;

* Creating empty data file to augment with each csv file;
DATA &permname.; RUN;

* Importing each excel file, starting with # for start and ending with # for end;
* Any file numbers missing in the sequence will generate no data sets (ignore errors);
%MACRO ImportExcel;
	%DO name=&start. %TO &end.;
		PROC IMPORT OUT=work.excel&name. DATAFILE="&folder.\&excelname." DBMS=XLSX REPLACE; 
		     GETNAMES=YES; SHEET="&name."; 
		RUN;
		* Now add subject ID, and move to first variable using RETAIN;
		DATA work.excel&name.;
			RETAIN Subj;
			SET work.excel&name.;
			Subj=&name.;
		RUN;
		* Now add to previous files (will be empty first time through);
		DATA &permname.; 
			SET &permname. work.excel&name.; 
		RUN;
	%END;
%MEND ImportExcel;
* Now call the macro to import all files from "start" to "end";
%ImportExcel;
